﻿Imports CnslLib
Imports System.Text
Imports System.Text.RegularExpressions

Module Program
    Dim line As String
    Dim o As CnslValidation


    Sub Main()

        line = loginExample()

    End Sub
    Function loginExample() As String
        Try
            Do
                '
                ' User input
                '
                line = Cnsl.ReadLine("User: ", True, minLength:=3)
                '
                ' On exit, line is minimum 3 charaters long (or a Function Key)
                '
                If line = CnslFns.functionKeys("Quit") Then
                    Exit Do
                ElseIf CnslFns.IsAFunctKey(line) Then
                    Continue Do ' Any other function key
                Else

                End If
                '
                ' Password
                '
                Dim validation As New CnslValidation(CnslValTypes.Password, True, 8, maxNumOfTries:=3, callback:=AddressOf ValidationCallback)
                line = Cnsl.ReadLine("Password: ", True, minLength:=3, validation:=validation)
                '
                ' If password valid, line is minimum 8 charaters long, contains a Capital Letter 
                ' and one of some special charactes (or is a Function Key) and, also, line="myPassword100%"
                '
                If CnslFns.IsNotAFunctKey(line) Then

                    CnslValidation.DisplayMsg("Successfully logged in!", ConsoleColor.Green, 1)
                    Exit Do

                ElseIf line = CnslFns.functionKeys("TriesExahusted") Then

                    CnslValidation.GetMessage(8, True)   ' tries limit exhausted
                    System.Threading.Thread.Sleep(1500)
                    line = CnslFns.functionKeys("Quit")
                    Exit Do

                End If
            Loop
        Catch ex As Exception
            CnslValidation.DisplayMsg($"Error {ex.Message}")
            Cnsl.ReadLine("", True, minLength:=0)
            line = CnslFns.functionKeys("Quit")
        End Try
        Return line
    End Function

    ' Note: All validation logic is kept in this file for demonstration purposes.
    ' This centralizes the example code for easier modification by the programmer.
    ' In a real-world application, you may consider moving specific validation logic
    ' or utility functions to separate files or modules for better organization and maintainability.
    ' However, the current structure allows for quick customization in one place.
    Function ValidationCallback(ByRef psw As StringBuilder, objVal As CnslValidation) As Boolean
        Dim ret As Boolean = False
        Try
            ' If arrived here 'psw' length is at minimum 8 characters long
            '
            CnslValidation.DisplayMsg(StrDup(79, " ")) ' clear possible previous messages
            Dim sPsw As String = psw.ToString
            Dim specialChars As String = "$%&/"
            objVal.maxNumOfTries += 1
            If Not Regex.IsMatch(sPsw, "[A-Z]+") Then
                CnslValidation.DisplayMsg("Input must contain as last one capital letter.")
                Exit Try
            ElseIf Not Regex.IsMatch(sPsw, "[0-9]+") Then
                CnslValidation.DisplayMsg("Input must contain as last one number.")
                Exit Try
            ElseIf Not Regex.IsMatch(sPsw, "[" + specialChars + "]+") Then
                CnslValidation.DisplayMsg("Input must contain as last one character like " + specialChars)
                Exit Try
            ElseIf sPsw <> "myPassword100%" Then
                ' Example password validation: 
                ' The fixed password "myPassword100%" is used for demonstration purposes.
                ' In a real-world application, passwords should be securely stored and retrieved from a database or another secure source.
                ' This is just a placeholder to show how password validation might work.
                objVal.maxNumOfTries -= 1 ' #tries only decrements when the previous conditions are verified
                Dim sN As String = IIf(objVal.maxNumOfTries > 1, "tries", "try")
                CnslValidation.DisplayMsg($"{objVal.maxNumOfTries } more " + sN + " left.")
                Exit Try
            End If
            Return True
        Catch ex As Exception
            Throw
        End Try
        Return ret
    End Function

End Module
